// ----------------------------------------------------------------------------
// General types
// ----------------------------------------------------------------------------

export interface IPoint {
	x: number;
	y: number;
}

// ----------------------------------------------------------------------------
// Runtime adapter interfaces
// ----------------------------------------------------------------------------
export enum RunTimeType {
	Bun = 'bun',
	Deno = 'deno',
	Unknown = 'common',
}

/**
 * The native functions for terminal settings
 */
export interface IFFI {
	/**
	 * Get the existing termios settings (for canonical mode)
	 */
	tcgetattr(fd: number, termiosPtr: unknown): number;

	/**
	 * Update the termios settings
	 */
	tcsetattr(fd: number, act: number, termiosPtr: unknown): number;

	/**
	 * Update the termios pointer with raw mode settings
	 */
	cfmakeraw(termiosPtr: unknown): void;

	/**
	 * Convert a TypedArray to an opaque pointer for ffi calls
	 */
	getPointer(ta: any): unknown;
}

export interface ITerminalSize {
	rows: number;
	cols: number;
}

/**
 * Runtime-specific IO streams
 */
export interface ITerminalIO {
	/**
	 * The generator function returning chunks of input from the stdin stream
	 */
	inputLoop(): AsyncGenerator<Uint8Array, void, unknown>;

	/**
	 * Get the size of the terminal
	 */
	getTerminalSize(): Promise<ITerminalSize>;

	/**
	 * Pipe a string to stdout
	 */
	write(s: string): Promise<void>;
}

export interface IRuntime {
	/**
	 * The name of the runtime
	 */
	name: RunTimeType;
	/**
	 * Runtime-specific FFI
	 */
	ffi: IFFI;
	/**
	 * Runtime-specific terminal functionality
	 */
	io: ITerminalIO;

	/**
	 * Set a beforeExit/beforeUnload event handler for the runtime
	 * @param cb - The event handler
	 */
	onExit(cb: () => void): void;

	/**
	 * Stop execution
	 *
	 * @param code
	 */
	exit(code?: number): void;
}

// ----------------------------------------------------------------------------
// Testing
// ----------------------------------------------------------------------------

/**
 * The shared test interface, so tests can be run by both runtimes
 */
export interface ITestBase {
	test(name: string, fn: () => void): void;
	assertEquals(actual: unknown, expected: unknown): void;
	assertNotEquals(actual: unknown, expected: unknown): void;
	assertStrictEquals(actual: unknown, expected: unknown): void;
	assertExists(actual: unknown): void;
	assertInstanceOf(actual: unknown, expectedType: any): void;
	assertTrue(actual: boolean): void;
	assertFalse(actual: boolean): void;
}
